module Lexoranking
  module Generators
    class InstallGenerator < ::Rails::Generators::Base
      include Rails::Generators::Migration
      source_root File.expand_path('../templates', __FILE__)
      desc "Add migration for Lexoranking models"
      class_option :model, type: :string, desc: 'Model name where to add rank column'
      RANKING_COLUMN = :rank

      def self.next_migration_number(path)
        next_migration_number = current_migration_number(path) + 1
        ActiveRecord::Migration.next_migration_number(next_migration_number)
      end

      def setup
        @model_name = options.model.pluralize
      end

      def copy_migrations
        validates_ranking_column!

        migration_template "migration.rb", "db/migrate/add_#{column}_to_#{model_name}.rb"
      end


      private
      attr_reader :model_name, :column

      def validates_ranking_column!
        model_exists! || _raise_model_do_not_exists

        @column = load_ranking_column(model_name.classify.constantize) || RANKING_COLUMN
      end

      def model_exists!
        ActiveRecord::Base.connection.tables.map(&:classify).include?(model_name.classify)
      end

      def load_ranking_column(model)
        model.ranking_column if model.respond_to? :ranking_column
      end

      def _raise_model_do_not_exists
        raise ::Lexoranking::ModelDoNotExists.new(
          "The model #{model_name.classify} do not exists",
          self
        )
      end
    end
  end
end
